// WinAVR 20030913

/**********************************************************
 * Registers used by GCC																	*
 **********************************************************
 * R0, R18-R27, R30-R31		can be used freely in assembler	*
 * R2-R17, R28-R29				must be pushed !								*
 * R1											must be zero !									*
 **********************************************************/

#include <avr/io.h>

	.section .text

	.stabs	"",100,0,0,div8u
	.stabs	"ascii.S",100,0,0,div8u

/***************************************************************************
 *
 * "div8u" - 8/8 Bit Unsigned Division
 *
 * This subroutine divides the two register variables "dd8u" (dividend) and 
 * "dv8u" (divisor). The result is placed in "dres8u" and the remainder in
 * "drem8u".
 *  
 * Number of words	:14
 * Number of cycles	:97
 * Low registers used	:1 (drem8u)
 * High registers used  :3 (dres8u/dd8u,dv8u,dcnt8u)
 *
 ***************************************************************************/
	.func		div8u

#define drem8u		r25					// remainder
#define dres8u		r26					// result
#define dd8u			r26					// dividend
#define dv8u			r27					// divisor
#define dcnt8u		r21					// loop counter


div8u:	sub		drem8u,drem8u		// clear remainder and carry
				ldi		dcnt8u,9				// init loop counter

d8u_1:	rol		dd8u						// shift left dividend
				dec		dcnt8u					// decrement counter
				brne	d8u_2						// if done
				ret										//  return

d8u_2:	rol		drem8u					// shift dividend into remainder
				sub		drem8u,dv8u			// remainder = remainder - divisor
				brcc	d8u_3						// if result negative
				add		drem8u,dv8u			// restore remainder
				clc										// clear carry to be shifted into result
				rjmp	d8u_1						// else

d8u_3:	sec										// set carry to be shifted into result
				rjmp	d8u_1

	.endfunc


//void bin2ascii8(u08 val, char *buf, u08 digits);
	.global	bin2ascii8
	.func		bin2ascii8

#define val				r24
#define buf_l			r22
#define buf_h			r23
#define	digits		r20

#define loop			r19
#define	tmp				r18

bin2ascii8:
	mov		ZL, buf_l
	mov		ZH, buf_h						// move the pointer ...
	mov		loop, digits				// loop counter
	
	subi	digits, 1						// digits -= 1
	add		ZL, digits
	adc		ZH, r1							// Z += digits
	
	ldi		dv8u, 10						// divide by 10, for conversion
	ldi		tmp, 0x30						// ascii offset
	mov		dd8u, val

bin2ascii8_1:
	rcall div8u								// divide
	
	add		drem8u, tmp					// add the ascii offset to the reminder
	st		Z, drem8u						// store the reminder
	
	subi	ZL, 1
	sbc		ZH, r1							// buf--
	
	dec		loop
	brne	bin2ascii8_1
	
	ret
	.endfunc


